try:
    import unittest2 as unittest
except ImportError:
    import unittest as unittest

from battlenet_client.util import *


class UtilityTests(unittest.TestCase):

    def test_localize(self):
        locale = localize('EnuS')
        self.assertIsInstance(locale, str)
        self.assertEqual(locale[:2], 'en')
        self.assertEqual(locale[-2:], 'US')

    def test_localize_wrong_type(self):
        self.assertRaises(TypeError, lambda: localize(1))
        self.assertRaises(TypeError, lambda: localize(['EnuS', 'EngB']))
        self.assertRaises(TypeError, lambda: localize(('EnuS', 'EngB')))
        self.assertRaises(TypeError, lambda: localize({'lang': 'en', 'country': 'US'}))

    def test_localize_wrong_lang_country(self):
        self.assertRaises(ValueError, lambda: localize('zzus'))
        self.assertRaises(ValueError, lambda: localize('enZZ'))

    def test_slugify(self):
        name = slugify("Zul'jin")
        self.assertEqual(name, 'zuljin')

    def test_slugify_case(self):
        name = slugify("Zul'jin")
        self.assertTrue(name.islower())

    def test_slugify_type_error(self):
        self.assertRaises(TypeError, lambda: slugify(1))
        self.assertRaises(TypeError, lambda: slugify(['EnuS', 'EngB']))
        self.assertRaises(TypeError, lambda: slugify(('EnuS', 'EngB')))
        self.assertRaises(TypeError, lambda: slugify({'lang': 'en', 'country': 'US'}))

    def test_slugify_no_apostrophe(self):
        name = slugify("Zul'jin")
        self.assertEqual(name.find('\''), -1)

    def test_slugify_no_space(self):
        name = slugify('Moon Guard')
        self.assertEqual(name.find(' '), -1)

    def test_currency_convertor_positive(self):
        money = currency_convertor(394567)
        self.assertIsInstance(money, (list, tuple))
        self.assertEqual(len(money), 3)
        self.assertEqual(money, (39, 45, 67))

    def test_currency_convertor_zero(self):
        money = currency_convertor('0')
        self.assertIsInstance(money, (list, tuple))
        self.assertEqual(len(money), 3)
        self.assertEqual(money, (0, 0, 0))

    def test_currency_convertor_negative(self):
        self.assertRaises(ValueError, lambda: currency_convertor(-1))

    def test_currency_convertor_wrong_types(self):
        self.assertRaises(TypeError, lambda: currency_convertor([1, 3, 4]))
        self.assertRaises(TypeError, lambda: currency_convertor((1, 3, 4)))
        self.assertRaises(TypeError, lambda: currency_convertor({'g': 1, 's': 3, 'c': 4}))



from decouple import config
from requests.exceptions import HTTPError

try:
    import unittest2 as unittest
except ImportError:
    import unittest

from battlenet_client.client import BattleNetClient


class CredentialClientTests(unittest.TestCase):

    def setUp(self):
        self.connection = BattleNetClient('us', "wow", config('CLIENT_ID'), config('CLIENT_SECRET'))

    def test_get_not_found(self):
        self.assertRaises(HTTPError,
                          lambda: self.connection.get(
                              f'{self.connection.api_host}/data/wow/playable-class/-1', 'en_US', 'static'))

    def test_get_found(self):
        self.assertIsInstance(self.connection.get(
                              f'{self.connection.api_host}/data/wow/playable-class/1', 'enus', 'static'), dict)

    def test_post_not_found(self):
        self.assertRaises(HTTPError,
                          lambda: self.connection.post(f'{self.connection.api_host}/data/wow/playable-class/-1',
                                                       'en_US', 'static'))

    def test_post_invalid(self):
        self.assertRaises(HTTPError,
                          lambda: self.connection.post(f'{self.connection.api_host}/data/wow/playable-class/1',
                                                       'enus', 'static'))

    def tearDown(self):
        self.connection.close()



class BattleNetError(Exception):
    pass


class BattleNetDataNotFoundError(BattleNetError):
    pass


class BattleNetRegionNotFoundError(BattleNetError):
    pass


class BattleNetNegativeIndexError(BattleNetError):
    pass


"""BattleNet Clients handle the processing on the requests with the Developer Portal API

.. moduleauthor: David "Gahd" Couples <gahdania@gahd.io>
"""

from io import BytesIO
from time import sleep
from urllib.parse import unquote

import requests
from oauthlib.oauth2 import BackendApplicationClient
from requests_oauthlib import OAuth2Session

from battlenet_client.util import localize
from battlenet_client.exceptions import BattleNetDataNotFoundError, BattleNetRegionNotFoundError


class BattleNetClient(OAuth2Session):
    """Handles the communication using OAuth v2 client to Battle.net REST API

    Args:
        region (str): region abbreviation for use with the APIs
        client_id (str): the client ID from the developer portal
        client_secret (str): the client secret from the developer portal

    Keyword Args:
        scope (list, optional): the scope or scopes to use during the endpoints that require the Web Application Flow
        redirect_uri (str, optional): the URI to return after a successful authentication between the user and Blizzard

    Attributes:
        tag (str): the region tag (abbreviation) of the client
        api_host (str): the host to use for accessing the API endpoints
        auth_host (str): the host to use for authentication
        render_host (str): the hose to use for images
    """

    def __init__(self, region, game, client_id, client_secret, *, scope=None, redirect_uri=None):

        self._state = None

        if region.strip().lower() in ('us', 'eu', 'kr', 'tw', 'cn'):
            self.tag = region.strip().lower()
        else:
            raise BattleNetRegionNotFoundError("Region not available")

        self.game = game

        self._client_secret = client_secret

        self.dynamic = f"dynamic-{self.tag}"
        self.static = f"static-{self.tag}"
        self.profile = f"profile-{self.tag}"

        if self.tag == 'cn':
            self.api_host = 'https://gateway.battlenet.com.cn'
            self.auth_host = 'https://www.battlenet.com.cn'
            self.render_host = None  # may need someone playing on the chinese client to help me
        elif self.tag == 'kr' or self.tag == 'tw':
            self.api_host = f'https://{self.tag}.api.blizzard.com'
            self.auth_host = f'https://apac.battle.net'
            self.render_host = f'https://render-{self.tag}.worldofwarcraft.com'
        else:
            self.api_host = f'https://{self.tag}.api.blizzard.com'
            self.auth_host = f'https://{self.tag}.battle.net'
            self.render_host = f'https://render-{self.tag}.worldofwarcraft.com'

        if redirect_uri and scope and 'openid' not in scope:
            self.auth_flow = 'oauth'
            super().__init__(client_id=client_id, scope=scope, redirect_uri=redirect_uri)
            # set the mode indicator of the client to "Web Application Flow"
        else:
            super().__init__(client=BackendApplicationClient(client_id=client_id))
            # set the mode indicator of the client to "Backend Application Flow"
            self.fetch_token()
            self.auth_flow = None

    def endpoint(self, uri, locale, namespace, **kwargs):

        kwargs['params'] = {'locale': localize(locale)}

        if 'headers' not in kwargs.keys():
            kwargs['headers'] = {'Battlenet-Namespace': getattr(self, namespace)}
        else:
            kwargs['headers']['Battlenet-Namespace'] = getattr(self, namespace)

        if 'fields' in kwargs.keys():
            kwargs['params'].update({key: value for key, value in kwargs['fields'].items()})
            kwargs.pop('fields', None)

        for _ in range(5):
            try:
                raw_data = super().get(uri, **kwargs)
                raw_data.raise_for_status()
            except requests.Timeout:
                sleep(2.5)
            except requests.exceptions.HTTPError as error:
                if error.response.status_code == 429:
                    sleep(1.0)
                elif error.response.status_code == 404:
                    raise BattleNetDataNotFoundError
                else:
                    raise error
            else:
                if raw_data.headers['content-type'].startswith('application/json'):
                    return raw_data.json()
                else:
                    return BytesIO(raw_data.content)

    def get_user_info(self, locale=None):
        """Returns the user info

        Args:
            locale (str): localization to use

        Returns:
            dict: the json decoded information for the user (user # and battle tag ID)
        """
        if not self.auth_flow:
            raise TypeError("Requires Authorization Workflow")

        url = f"{self.auth_host}/oauth/userinfo"
        return self.post(url, params={'locale': locale}, headers={'Battlenet-Namespace': None})

    def validate_token(self):
        """Checks with the API if the token is good or not.

        Returns:
            bool: True of the token is valid, false otherwise.
        """
        url = f"{self.auth_host}/oauth/check_token"
        data = super().post(url, params={'token': self.access_token}, headers={'Battlenet-Namespace': None})
        return data.status_code == 200 and data.json()['client_id'] == self.client_id

    def authorization_url(self, **kwargs):
        """Prepares and returns the authorization URL to the Battle.net authorization servers

        Returns:
            str: the URL to the Battle.net authorization server
        """
        if not self.auth_flow:
            raise ValueError("Requires Authorization Workflow")

        auth_url = f"{self.auth_host}/oauth/authorize"
        authorization_url, self._state = super().authorization_url(url=auth_url, **kwargs)
        return unquote(authorization_url)

    def fetch_token(self, **kwargs):
        token_url = f"{self.auth_host}/oauth/token"
        super().fetch_token(token_url=token_url, client_id=self.client_id, client_secret=self._client_secret,
                            **kwargs)



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

# noinspection PyPackageRequirements
setup(
    name="battlenet_client",
    version="1.2.0b18",
    author="David \"Gahd\" Couples",
    author_email="gahdania@gahd.io",
    description="Battle.net REST API Connections",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/battlenet1/bnet",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Operating System :: OS Independent",
    ],
    install_requires=['requests_oauthlib', 'python-decouple'],
    python_requires='>=3.8',
)
